import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';
import 'package:file_picker/file_picker.dart';
import 'package:get/get_connect/http/src/request/request.dart';
import 'package:http/http.dart' as Http;
import 'package:get/get.dart';
import 'package:image_picker/image_picker.dart';
import 'package:lms_user_app/core/common_model/errors_model.dart';
import 'package:lms_user_app/core/helper/help_me.dart';
import 'package:lms_user_app/utils/app_constants.dart';
import 'package:flutter/foundation.dart' as Foundation;
import 'package:lms_user_app/utils/dev_util.dart';
import 'package:path/path.dart';
import 'package:shared_preferences/shared_preferences.dart';

class ApiClient extends GetxService {
  final String? appBaseUrl;
  final SharedPreferences sharedPreferences;
  static final String noInternetMessage = 'connection_to_api_server_failed'.tr;
  final int timeoutInSeconds = 30;

  String? token;
  late Map<String, String> _mainHeaders;

  //remove this token when app ready to publish
  String testToken = 'eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJodHRwczovL2ZhY3VsdHkuc3BhZ3JlZW4ubmV0L2RlbW8vYXBpL2xvZ2luIiwiaWF0IjoxNzAwMzkwMDUwLCJuYmYiOjE3MDAzOTAwNTAsImp0aSI6Imhja1dzZXdZQXdGdk1NVFoiLCJzdWIiOiIyIiwicHJ2IjoiMjNiZDVjODk0OWY2MDBhZGIzOWU3MDFjNDAwODcyZGI3YTU5NzZmNyJ9.qVf4-uvtGsrsT1SLQLPVepzihUlrca-KEs3JvNB7Mtw';

  ApiClient({required this.appBaseUrl, required this.sharedPreferences}) {
    token = sharedPreferences.getString(AppConstants.token) ?? testToken;
    printLog("-------apiClient constructor: token: $token");

    appLog(tag: "API-CLIENT token", msg: token);
    initApiClient();
  }

  void initApiClient() {
    ///pick zone id to update header
    updateHeader(
      token,
      sharedPreferences.getString(AppConstants.languageCode),
    );
  }

  void updateHeader(String? token, String? languageCode) {
    _mainHeaders = {
      'Content-Type': 'application/json; charset=UTF-8',
      AppConstants.localizationKey:
          languageCode ?? AppConstants.languages[0].languageCode!,
      'Authorization': 'Bearer $token'
    };
  }

  Future<Response> getData(String uri,
      {Map<String, dynamic>? query, Map<String, String>? headers}) async {

    appLog(tag: "GET URL", msg: appBaseUrl!+uri);

    try {
      Http.Response response = await Http.get(
        Uri.parse(appBaseUrl! + uri),
        headers: headers ?? _mainHeaders,
      ).timeout(Duration(seconds: timeoutInSeconds));

      return handleResponse(response, uri);
    } catch (e) {
      return Response(statusCode: 1, statusText: noInternetMessage);
    }
  }

  Future<Response> postData(String uri, dynamic body,
      {Map<String, String>? headers}) async {

    appLog(tag: "POST URL", msg: appBaseUrl!+uri);

    Http.Response response = await Http.post(
      Uri.parse(appBaseUrl! + uri),
      body: jsonEncode(body),
      headers: headers ?? _mainHeaders,
    ).timeout(Duration(seconds: timeoutInSeconds));
    try {
      return handleResponse(response, uri);
    } catch (e) {
      return Response(statusCode: 1, statusText: noInternetMessage);
    }
  }

  Future<Response> postMultipartDataConversation(
      String? uri, Map<String, String> body, List<MultipartBody>? multipartBody,
      {Map<String, String>? headers, PlatformFile? otherFile}) async {
    Http.MultipartRequest _request =
        Http.MultipartRequest('POST', Uri.parse(appBaseUrl! + uri!));
    _request.headers.addAll(headers ?? _mainHeaders);

    if (otherFile != null) {
      _request.files.add(Http.MultipartFile('files[${multipartBody!.length}]',
          otherFile.readStream!, otherFile.size,
          filename: basename(otherFile.name)));
    }
    if (multipartBody != null) {
      for (MultipartBody multipart in multipartBody) {
        Uint8List _list = await multipart.file.readAsBytes();
        _request.files.add(Http.MultipartFile(
          multipart.key!,
          multipart.file.readAsBytes().asStream(),
          _list.length,
          filename: '${DateTime.now().toString()}.png',
        ));
      }
    }
    _request.fields.addAll(body);
    Http.Response _response =
        await Http.Response.fromStream(await _request.send());
    return handleResponse(_response, uri);
  }

  Future<Response> postMultipartData(
      String? uri, Map<String, String> body, List<MultipartBody>? multipartBody,
      {Map<String, String>? headers}) async {
    try {
      Http.MultipartRequest _request =
          Http.MultipartRequest('POST', Uri.parse(appBaseUrl! + uri!));
      _request.headers.addAll(headers ?? _mainHeaders);
      for (MultipartBody multipart in multipartBody!) {
        File _file = File(multipart.file.path);
        _request.files.add(Http.MultipartFile(
          multipart.key!,
          _file.readAsBytes().asStream(),
          _file.lengthSync(),
          filename: _file.path.split('/').last,
        ));
      }
      _request.fields.addAll(body);
      Http.Response _response =
          await Http.Response.fromStream(await _request.send());
      return handleResponse(_response, uri);
    } catch (e) {
      return Response(statusCode: 1, statusText: noInternetMessage);
    }
  }

  Future<Response> putData(String? uri, dynamic body,
      {Map<String, String>? headers}) async {
    try {
      Http.Response _response = await Http.put(
        Uri.parse(appBaseUrl! + uri!),
        body: jsonEncode(body),
        headers: headers ?? _mainHeaders,
      ).timeout(Duration(seconds: timeoutInSeconds));
      return handleResponse(_response, uri);
    } catch (e) {
      return Response(statusCode: 1, statusText: noInternetMessage);
    }
  }

  Future<Response> deleteData(String? uri,
      {Map<String, String>? headers}) async {
    try {
      Http.Response _response = await Http.delete(
        Uri.parse(appBaseUrl! + uri!),
        headers: headers ?? _mainHeaders,
      ).timeout(Duration(seconds: timeoutInSeconds));
      return handleResponse(_response, uri);
    } catch (e) {
      return Response(statusCode: 1, statusText: noInternetMessage);
    }
  }

  Response handleResponse(Http.Response response, String? uri) {
    dynamic _body;
    try {
      _body = jsonDecode(response.body);
    } catch (e) {}
    Response _response = Response(
      body: _body ?? response.body,
      bodyString: response.body.toString(),
      request: Request(
          headers: response.request!.headers,
          method: response.request!.method,
          url: response.request!.url),
      headers: response.headers,
      statusCode: response.statusCode,
      statusText: response.reasonPhrase,
    );
    if (_response.statusCode != 200 &&
        _response.body != null &&
        _response.body is! String) {
      if (_response.body.toString().startsWith('{response_code:')) {
        ErrorsModel _errorResponse = ErrorsModel.fromJson(_response.body);
        _response = Response(
            statusCode: _response.statusCode,
            body: _response.body,
            statusText: _errorResponse.responseCode);
      } else if (_response.body.toString().startsWith('{message')) {
        _response = Response(
            statusCode: _response.statusCode,
            body: _response.body,
            statusText: _response.body['message']);
      }
    } else if (_response.statusCode != 200 && _response.body == null) {
      _response = Response(statusCode: 0, statusText: noInternetMessage);
    }
    if (Foundation.kDebugMode) {
      //debugPrint('====> API Response: [${_response.statusCode}] $uri\n${_response.body}');
    }
    return _response;
  }
}

class MultipartBody {
  String? key;
  XFile file;

  MultipartBody(this.key, this.file);
}
